import io
import os.path
import unittest
from unittest.mock import patch

from drb.core import ParsedPath
from drb.exceptions.core import DrbException

from drb.drivers.aws3 import Auth, DrbS3Service, Requests, DrbS3Object
from tests.utility import S3Obj, DownloadObject, start_mock, stop_mock


class TestS3Object(unittest.TestCase):
    service = 's3'
    auth = Auth(service_name=service)
    node = DrbS3Service(auth)
    obj = S3Obj('tests', None)
    with patch.object(Requests, 'list_buckets') as mock:
        mock.return_value = [S3Obj('my_bucket', None),
                             S3Obj('my_bucket_2', None)]
        bucket_1 = node['my_bucket']
    node = DrbS3Object(obj, bucket_1)

    @classmethod
    def setUpClass(cls) -> None:
        start_mock('http://my_tmp_url/')

    @classmethod
    def tearDownClass(cls) -> None:
        stop_mock()

    def test_name(self):
        self.assertEqual('tests', self.node.name)

    def test_children(self):
        self.assertIsNotNone(self.node.children)
        self.assertIsInstance(self.node.children, list)
        self.assertEqual(0, len(self.node.children))

    def test_has_child(self):
        self.assertFalse(self.node.has_child('tests.txt'))
        self.assertFalse(self.node.has_child())
        self.assertFalse(self.node.has_child('test.txt', 'ns'))

    def test_attributes(self):
        self.assertIsNotNone(self.node.attributes)
        self.assertEqual(2, len(self.node.attributes))

    def test_get_attribute(self):
        for attr in self.node.attributes:
            self.assertIsNotNone(self.node.get_attribute(attr[0]))
            self.assertIsNotNone(self.node @ attr[0])
        with self.assertRaises(DrbException):
            self.node.get_attribute('wrong_name')
        with self.assertRaises(KeyError):
            self.node.get_attribute(self.node.attributes[0], 'namespace')

    def test_parent(self):
        self.assertEqual(self.bucket_1, self.node.parent)

    def test_path(self):
        self.assertEqual(ParsedPath('s3/my_bucket/tests').path,
                         self.node.path.path)

    @patch.object(Requests, 'get_obj',
                  return_value=io.BytesIO(b'This is for testing.'))
    def test_impl(self, mock):
        self.assertFalse(self.node.has_impl(DrbException))
        self.assertTrue(self.node.has_impl(io.BytesIO))
        with self.assertRaises(DrbException):
            self.node.get_impl(DrbException)
        with self.node.get_impl(io.BytesIO) as stream:
            self.assertEqual(b'This is for testing.',
                             stream.read())

    @patch.object(Requests, 'get_obj',
                  return_value=io.BytesIO(b'This is for testing.'))
    def test_impl_read(self, mock):
        with self.node.get_impl(io.BytesIO) as stream:
            self.assertEqual(b'Th',
                             stream.read(2))
            self.assertEqual(b'is i',
                             stream.read(4))

    @patch.object(Requests, 'get_obj',
                  return_value=io.BytesIO(b'This is for testing.'))
    def test_impl_tell(self, mock):
        with self.node.get_impl(io.BytesIO) as stream:
            self.assertEqual(stream.seek(10), 10)
            self.assertEqual(b'r t', stream.read()[:3])
            self.assertEqual(stream.tell(), len('This is for testing.'))

    @patch.object(Requests, 'get_temp_url',
                  return_value='http://my_tmp_url/')
    def test_tmp_url(self, mock):
        with self.node.get_impl(io.BytesIO, temp_url=True) as stream:
            self.assertEqual('This is for testing.',
                             stream.read().decode())

    def test_name_space_uri(self):
        self.assertIsNone(self.node.namespace_uri)

    def test_value(self):
        self.assertIsNone(self.node.value)

    def test_auth(self):
        self.assertEqual(self.auth, self.node.get_auth())



import io
from abc import ABC
from typing import List

import boto3
from botocore.client import BaseClient
from deprecated.classic import deprecated
from drb.core import DrbNode, DrbFactory
from drb.drivers.http import DrbHttpNode
from drb.nodes.abstract_node import AbstractNode
from requests.auth import HTTPBasicAuth


class Auth:
    """
    This class give us all the requirement to connect to a swift service.

    Parameters:
        service_name: The name of a service, e.g. 's3' or 'ec2'.
            region_name: The name of the region associated with the client.
            A client is associated with a single region.

        api_version: The API version to use.  By default, this implementation
            use the latest API version. You only need to specify this parameter
            if you want to use a previous API version of the client.

        use_ssl: Whether to use SSL.  By default, SSL is used.
            Note that not all services support non-ssl connections.

        verify: Whether to verify SSL certificates. By default,
            SSL certificates are verified.  You can provide the following
            values:
            * False - do not validate SSL certificates.  SSL will still be
            used (unless use_ssl is False), but SSL certificates
            will not be verified.
            * path/to/cert/bundle.pem - A filename of the CA cert bundle to
            uses.

        endpoint_url: The complete URL to use for the constructed
            client. If this value is provided, then ``use_ssl`` is ignored.

        aws_access_key_id: The access key to use when creating
            the client.

        aws_secret_access_key: The secret key to use when creating
            the client.

        aws_session_token: The session token to use when creating
            the client.

        config: Advanced client configuration options. If region_name
            is specified in the client config, its value will take precedence
            over environment variables and configuration values, but not over
            a region_name value passed explicitly to the method.  If
            user_agent_extra is specified in the client config, it overrides
            the default user_agent_extra provided by the resource API.
      """

    def __init__(self,
                 **kwargs):
        self.service_name = "s3"
        self.region_name = kwargs.get("", None)
        self.api_version = kwargs.get("api_version", None)
        self.use_ssl = kwargs.get("use_ssl", True)
        self.verify = kwargs.get("verify", None)
        self.endpoint_url = kwargs.get("endpoint_url", None)
        self.aws_access_key_id = kwargs.get("aws_access_key_id", None)
        self.aws_secret_access_key = kwargs.get("aws_secret_access_key", None)
        self.aws_session_token = kwargs.get("aws_session_token", None)
        self.config = kwargs.get("config", None)

    def get_session(self):
        session = boto3.session.Session()

        return session.client(
            service_name=self.service_name,
            region_name=self.region_name,
            api_version=self.api_version,
            use_ssl=self.use_ssl,
            verify=self.verify,
            endpoint_url=self.endpoint_url,
            aws_access_key_id=self.aws_access_key_id,
            aws_secret_access_key=self.aws_secret_access_key,
            aws_session_token=self.aws_session_token,
            config=self.config
        )


class Connection:
    """
    This class use the singleton pattern to provide too
    much connection to the s3 server.

    Parameters:
        auth: An Auth object to provide all the information required
              to establish the connection with the server.
    """

    conn = None

    def __new__(cls, auth: Auth) -> BaseClient:
        if cls.conn is None:
            cls.conn = boto3.resource(
                service_name=auth.service_name,
                region_name=auth.region_name,
                api_version=auth.api_version,
                use_ssl=auth.use_ssl,
                verify=auth.verify,
                endpoint_url=auth.endpoint_url,
                aws_access_key_id=auth.aws_access_key_id,
                aws_secret_access_key=auth.aws_secret_access_key,
                aws_session_token=auth.aws_session_token,
                config=auth.config
            )
        return cls.conn


class Requests:
    def __init__(self, auth: Auth):
        self.conn = Connection(auth)
        self.auth = auth

    def list_buckets(self):
        return self.conn.buckets.all()

    def list_objects(self, name):
        return self.conn.Bucket(name).objects.all()

    def get_temp_url(self, bucket, name):
        s3_api = self.auth.get_session()
        return s3_api.generate_presigned_url(
            'get_object', Params={
                'Bucket': bucket,
                'Key': name
            },
            ExpiresIn=3600
        )

    def get_obj(self, bucket, name):
        buff = io.BytesIO()
        obj = self.conn.Object(bucket, name)
        obj.download_fileobj(buff)
        return buff


def check_args(*args):
    return len(args) > 0 and isinstance(
        args[0],
        int
    ) and args[0] > 0


class DrbS3Node(AbstractNode, ABC):
    """
    Common DrbS3Node interface
    """

    def __init__(self, auth: Auth):
        super(DrbS3Node, self).__init__()
        self._auth = auth
        self._conn = None

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError

    def close(self) -> None:
        """
        Close The s3 connection
        """
        if self._conn is not None:
            self._conn.close()

    def get_auth(self) -> Auth:
        """
        Return the Auth object created to access the service.
        """
        return self._auth

    def __eq__(self, other):
        return isinstance(other, DrbS3Node) and \
            self._auth == other._auth

    def __hash__(self):
        return hash(self._auth)


class DrbS3Object(DrbS3Node):
    def __init__(self, obj, parent: DrbS3Node):
        super().__init__(auth=parent.get_auth())
        self.req = Requests(parent.get_auth())
        self._obj = obj
        self.name = obj.key
        self.parent = parent
        self.add_impl(io.BytesIO, self._get_stream)
        self._init_attrs()

    def _init_attrs(self):
        for e in self._obj.get_available_subresources():
            name = e
            value = self._obj.key
            self.__imatmul__((name, value))

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    def children(self) -> List[DrbNode]:
        return []

    @staticmethod
    def _get_stream(node: DrbNode, **kwargs):
        if not isinstance(node, DrbS3Object):
            raise TypeError(f'Invalid node type: {type(node)}')

        if kwargs.get('temp_url', False):
            tmp_url = node.req.get_temp_url(node.parent.name, node.name)
            return DrbHttpNode(tmp_url).get_impl(io.BytesIO)
        else:
            return node.req.get_obj(node.parent.name, node.name)


class DrbS3Bucket(DrbS3Node):

    def __init__(self, obj, parent: DrbS3Node):
        super().__init__(auth=parent.get_auth())
        self.req = Requests(parent.get_auth())
        self._bucket = obj
        self.name = obj.name
        self._children = None
        self.parent = parent
        self._init_attrs()

    def _init_attrs(self):
        for e in self._bucket.get_available_subresources():
            name = e
            value = self._bucket.name
            self.__imatmul__((name, value))

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    def children(self) -> List[DrbNode]:
        self._conn = Connection(self._auth)
        if self._children is None:
            self._children = []
            objects = self.req.list_objects(self.name)
            for obj in objects:
                self._children.append(DrbS3Object(obj, self))

        return self._children


class DrbS3Service(DrbS3Node):
    def __init__(self, auth: Auth):
        super().__init__(auth=auth)
        self.req = Requests(auth)
        self._children = None
        self.name = self._auth.service_name

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    def children(self) -> List[DrbNode]:
        self._conn = Connection(self._auth)
        if self._children is None:
            buckets = self.req.list_buckets()
            self._children = [
                DrbS3Bucket(bucket, self)
                for bucket in buckets
            ]
        return self._children


class S3NodeFactory(DrbFactory):

    def _create(self, node: DrbNode) -> DrbNode:
        if isinstance(node, DrbS3Node):
            return node
        if isinstance(node, DrbHttpNode):
            if isinstance(node.auth, HTTPBasicAuth):
                auth = Auth(service_name=node.path.original_path,
                            aws_access_key_id=node.auth.username,
                            aws_secret_access_key=node.auth.password)
            else:
                auth = Auth(service_name=node.path.original_path)
            return DrbS3Service(auth=auth)
        raise NotImplementedError("Call impl method")



import unittest
import uuid

from drb.core.factory import FactoryLoader
from drb.nodes.logical_node import DrbLogicalNode
from drb.topics.dao import ManagerDao
from drb.topics.topic import TopicCategory

from drb.drivers.aws3 import S3NodeFactory


class TestSwiftSignature(unittest.TestCase):
    s3_id = uuid.UUID('4ab73f92-bbff-11ec-8422-0242ac120002')
    storage_url = "https+s3://my.s3/"
    storage_url2 = "http+s3://my.s3/"
    storage_url3 = "https://my.s2/"
    fc_loader = None
    topic_loader = None

    @classmethod
    def setUpClass(cls) -> None:
        cls.fc_loader = FactoryLoader()
        cls.topic_loader = ManagerDao()

    def test_impl_loading(self):
        factory_name = 'aws3'

        factory = self.fc_loader.get_factory(factory_name)
        self.assertIsNotNone(factory)
        self.assertIsInstance(factory, S3NodeFactory)

        topic = self.topic_loader.get_drb_topic(self.s3_id)
        self.assertIsNotNone(factory)
        self.assertEqual(self.s3_id, topic.id)
        self.assertEqual('Amazon Simple Storage Service', topic.label)
        self.assertIsNone(topic.description)
        self.assertEqual(TopicCategory.CONTAINER, topic.category)
        self.assertEqual(factory_name, topic.factory)

    def test_impl_signatures(self):
        topic = self.topic_loader.get_drb_topic(self.s3_id)

        node = DrbLogicalNode(self.storage_url)
        self.assertTrue(topic.matches(node))

        node = DrbLogicalNode(self.storage_url2)
        self.assertTrue(topic.matches(node))

        node = DrbLogicalNode(self.storage_url3)
        self.assertFalse(topic.matches(node))



import unittest

from drb.core import DrbNode
from drb.drivers.http import DrbHttpNode

from drb.drivers.aws3 import S3NodeFactory, Auth, DrbS3Service


class TestSwiftFactory(unittest.TestCase):
    node = None
    auth = None
    storage_url = "https://my.s3/"
    options = {
        'aws_key_id': 'Default',
        'aws_secret_key': 'Default',
    }

    def test_create(self):
        factory = S3NodeFactory()
        auth = Auth(service_name=self.storage_url,
                    aws_access_key_id=self.options['aws_key_id'],
                    aws_secret_access_key=self.options['aws_secret_key'])
        node = DrbS3Service(auth=auth)
        node = factory.create(node)
        self.assertIsInstance(node, (DrbS3Service, DrbNode))

        node = DrbHttpNode(self.storage_url)
        node = factory.create(node)
        self.assertIsInstance(node, (DrbS3Service, DrbNode))



import os.path
import unittest
from unittest.mock import patch

from drb.core import ParsedPath
from drb.exceptions.core import DrbException

from drb.drivers.aws3 import Auth, DrbS3Service, Requests
from tests.utility import S3Obj


class TestSwiftBucket(unittest.TestCase):
    service = 's3'
    auth = Auth(service_name=service)
    node = DrbS3Service(auth)
    bucket_with_obj = S3Obj('my_bucket', None)
    bucket_no_obj = S3Obj('my_bucket_2', None)
    with patch.object(Requests, 'list_buckets') as mock:
        mock.return_value = [S3Obj('my_bucket', None),
                             S3Obj('my_bucket_2', None)]
        bucket_1 = node['my_bucket']
        bucket_2 = node['my_bucket_2']
        bucket_2._children = []

    @patch.object(Requests, 'list_buckets',
                  return_value=[bucket_no_obj, bucket_with_obj])
    def test_name(self, mock_children):
        self.assertEqual('my_bucket', self.node['my_bucket'].name)
        self.assertEqual('my_bucket_2', self.node['my_bucket_2'].name)

    @patch.object(Requests, 'list_objects',
                  return_value=[S3Obj('tests', bucket_1)])
    def test_children(self, mock_children):
        self.assertIsNotNone(self.node['my_bucket'].children)
        self.assertIsInstance(self.node['my_bucket'].children, list)
        self.assertEqual(1, len(self.node['my_bucket'].children))
        self.assertEqual(0, len(self.node['my_bucket_2'].children))

    def test_attributes(self):
        self.assertIsNotNone(self.node['my_bucket'].attributes)
        self.assertEqual(2, len(self.node['my_bucket'].attributes))

    def test_get_attribute(self):
        for attr in self.node['my_bucket'].attributes:
            self.assertIsNotNone(self.node['my_bucket'].get_attribute(attr[0]))
            self.assertIsNotNone(self.node['my_bucket'] @ attr[0])
        with self.assertRaises(DrbException):
            self.node['my_bucket'].get_attribute('wrong_name')
        with self.assertRaises(DrbException):
            self.node['my_bucket'] @ 'wrong_name'
        with self.assertRaises(KeyError):
            self.node['my_bucket'].get_attribute(
                self.node['my_bucket'].attributes[0], 'namespace')

    def test_parent(self):
        self.assertEqual(self.node, self.node['my_bucket'].parent)

    def test_path(self):
        self.assertEqual(ParsedPath('s3/my_bucket').path,
                         self.node['my_bucket'].path.path)

    def test_impl(self):
        self.assertFalse(self.node.has_impl('impl'))
        with self.assertRaises(DrbException):
            self.node.get_impl('impl')

    def test_has_child(self):
        self.assertTrue(self.node['my_bucket'].has_child('tests'))
        self.assertTrue(self.node['my_bucket'].has_child())
        self.assertFalse(self.node['my_bucket'].has_child('not_here'))
        self.assertFalse(self.node['my_bucket'].has_child('my_bucket', 'ns'))
        self.assertFalse(self.node['my_bucket_2'].has_child())

    def test_name_space_uri(self):
        self.assertIsNone(self.node.namespace_uri)

    def test_value(self):
        self.assertIsNone(self.node.value)

    def test_auth(self):
        self.assertEqual(self.auth, self.node.get_auth())



import os.path
import unittest
from unittest.mock import patch

from drb.core import ParsedPath
from drb.exceptions.core import DrbException

from drb.drivers.aws3 import Auth, DrbS3Service, Requests
from tests.utility import S3Obj


class TestSwiftService(unittest.TestCase):
    service = 's3'
    auth = Auth(service_name=service, )
    node = DrbS3Service(auth)
    children = [
        S3Obj('my_bucket_1', node),
        S3Obj('my_bucket_2', node)
    ]

    def test_name(self):
        self.assertEqual('s3', self.node.name)

    def test_eq(self):
        self.assertEqual(self.node, DrbS3Service(self.auth))
        self.assertEqual(self.node.__hash__(),
                         DrbS3Service(self.auth).__hash__())

    @patch.object(Requests, 'list_buckets', return_value=children)
    def test_children(self, mock_children):
        self.assertIsNotNone(self.node.children)
        self.assertIsInstance(self.node.children, list)
        self.assertEqual(2, len(self.node.children))

    @patch.object(Requests, 'list_buckets', return_value=children)
    def test_has_child(self, mock_children):
        self.assertTrue(self.node.has_child('my_bucket_1'))
        self.assertTrue(self.node.has_child())
        self.assertFalse(self.node.has_child('not_here'))
        self.assertFalse(self.node.has_child('my_bucket_1', 'ns'))

    def test_attributes(self):
        self.assertIsNotNone(self.node.attributes)
        self.assertEqual(0, len(self.node.attributes))

    def test_parent(self):
        self.assertIsNone(self.node.parent)

    def test_path(self):
        self.assertEqual(ParsedPath('s3').path, self.node.path.path)

    def test_impl(self):
        self.assertFalse(self.node.has_impl('impl'))
        with self.assertRaises(DrbException):
            self.node.get_impl('impl')

    def test_name_space_uri(self):
        self.assertIsNone(self.node.namespace_uri)

    def test_value(self):
        self.assertIsNone(self.node.value)

    def test_auth(self):
        self.assertEqual(self.auth, self.node.get_auth())
